using System.Collections.Generic;
using OpenBveApi.Hosts;

namespace LibRender2.Text
{
	public partial class Fonts
	{
		/// <summary>Gets the list of blocked fonts</summary>
		public static List<string> BlockedFonts
		{
			get
			{
				switch (currentHost.Platform)
				{
					case HostPlatform.MicrosoftWindows:
						return windowsBlockedFonts;
					case HostPlatform.FreeBSD:
					case HostPlatform.GNULinux:
					case HostPlatform.WINE:
						return linuxBlockedFonts;
					case HostPlatform.AppleOSX:
						return appleBlockedFonts;
				}

				return new List<string>();
			}
		}

		private static readonly List<string> windowsBlockedFonts = new List<string>
		{
			"Aharoni", // no regular variant, so crashes
			"Bookshelf Symbol 7",
			"Cambria Math",
			"HoloLens MDL2 Assets",
			"Marlett",
			"MS Outlook",
			"MS Reference Sans Serif",
			"MS Reference Specialty",
			"MT Extra",
			"Segoe MDL2 Assets",
			"Symbol",
			"Wingdings",
			"Wingdings 2",
			"Wingdings 3",
			"Webdings",
			"ZWAdobeF"
		};

		private static readonly List<string> linuxBlockedFonts = new List<string>
		{
			/*
			 * Most of the Noto font set hasn't actually even got a full Western glyph set, so ignore them completely, as otherwise we get junk everywhere
			 */
			"DejaVuMathTeXGyre-Regular",
			"Dingbats",
			"Droid Sans Fallback",
			"Noto Color Emoji",
			"Noto Emoji",
			"Noto Kufi Arabic",
			"Noto Music",
			"Noto Naskh Arabic UI",
			"Noto Naskh Arabic",
			"Noto Nastaliq Urdu",
			"Noto Sans Adlam Unjoined",
			"Noto Sans Adlam",
			"Noto Sans Anatolian Hieroglyphs",
			"Noto Sans Arabic UI",
			"Noto Sans Arabic",
			"Noto Sans Armenian",
			"Noto Sans Avestan",
			"Noto Sans Bamum",
			"Noto Sans Bassa Vah",
			"Noto Sans Batak",
			"Noto Sans Bengali UI",
			"Noto Sans Bengali",
			"Noto Sans Bhaiksuki",
			"Noto Sans Brahmi",
			"Noto Sans Buginese",
			"Noto Sans Buhid",
			"Noto Sans Canadian Aboriginal",
			"Noto Sans Carian",
			"Noto Sans Caucasian Albanian",
			"Noto Sans Chakma",
			"Noto Sans Cham",
			"Noto Sans Cherokee",
			"Noto Sans Coptic",
			"Noto Sans Cuneiform",
			"Noto Sans Cypriot",
			"Noto Sans Deseret",
			"Noto Sans Devanagari UI",
			"Noto Sans Devanagari",
			"Noto Sans Display",
			"Noto Sans Duployan",
			"Noto Sans Egyptian Hieroglyphs",
			"Noto Sans Elbasan",
			"Noto Sans Ethiopic",
			"Noto Sans Georgian",
			"Noto Sans Glagolitic",
			"Noto Sans Gothic",
			"Noto Sans Grantha",
			"Noto Sans Gujarati UI",
			"Noto Sans Gujarati",
			"Noto Sans Gurmukhi UI",
			"Noto Sans Gurmukhi",
			"Noto Sans Hanunoo",
			"Noto Sans Hatran",
			"Noto Sans Hebrew",
			"Noto Sans Imperial Aramaic",
			"Noto Sans Inscriptional Pahlavi",
			"Noto Sans Inscriptional Parthia",
			"Noto Sans Javanese",
			"Noto Sans Kaithi",
			"Noto Sans Kannada UI",
			"Noto Sans Kannada",
			"Noto Sans Kayah Li",
			"Noto Sans Kharoshthi",
			"Noto Sans Khmer UI",
			"Noto Sans Khmer",
			"Noto Sans Khojki",
			"Noto Sans Khudawadi",
			"Noto Sans Lao UI",
			"Noto Sans Lao",
			"Noto Sans Lepcha",
			"Noto Sans Limbu",
			"Noto Sans Linear A",
			"Noto Sans Linear B",
			"Noto Sans Lisu",
			"Noto Sans Lycian",
			"Noto Sans Lydian",
			"Noto Sans Mahajani",
			"Noto Sans Malayalam UI",
			"Noto Sans Malayalam",
			"Noto Sans Mandaic",
			"Noto Sans Manichaean",
			"Noto Sans Marchen",
			"Noto Sans Math",
			"Noto Sans Meetei Mayek",
			"Noto Sans Mende Kikakui",
			"Noto Sans Meroitic",
			"Noto Sans Miao",
			"Noto Sans Modi",
			"Noto Sans Mongolian",
			"Noto Sans Mro",
			"Noto Sans Multani",
			"Noto Sans Myanmar UI",
			"Noto Sans Myanmar",
			"Noto Sans Nabataean",
			"Noto Sans New Tai Lue",
			"Noto Sans Newa",
			"Noto Sans NKo",
			"Noto Sans Ogham",
			"Noto Sans Ol Chiki",
			"Noto Sans Old Hungarian",
			"Noto Sans Old Italic",
			"Noto Sans Old North Arabian",
			"Noto Sans Old Permic",
			"Noto Sans Old Persian",
			"Noto Sans Old South Arabian",
			"Noto Sans Old Turkic",
			"Noto Sans Oriya UI",
			"Noto Sans Oriya",
			"Noto Sans Osage",
			"Noto Sans Osmanya",
			"Noto Sans Pahawh Hmong",
			"Noto Sans Palmyrene",
			"Noto Sans Pau Cin Hau",
			"Noto Sans PhagsPa",
			"Noto Sans Phoenician",
			"Noto Sans Psalter Pahlavi",
			"Noto Sans Rejang",
			"Noto Sans Runic",
			"Noto Sans Samaritan",
			"Noto Sans Saurashtra",
			"Noto Sans Sharada",
			"Noto Sans Shavian",
			"Noto Sans Sinhala UI",
			"Noto Sans Sinhala",
			"Noto Sans Sora Sompeng",
			"Noto Sans Sundanese",
			"Noto Sans Syloti Nagri",
			"Noto Sans Symbols",
			"Noto Sans Symbols2",
			"Noto Sans Syriac Eastern",
			"Noto Sans Syriac Estrangela",
			"Noto Sans Syriac Western",
			"Noto Sans Syriac",
			"Noto Sans Tagalog",
			"Noto Sans Tagbanwa",
			"Noto Sans Tai Le",
			"Noto Sans Tai Tham",
			"Noto Sans Tai Viet",
			"Noto Sans Takri",
			"Noto Sans Tamil UI",
			"Noto Sans Tamil",
			"Noto Sans Telugu UI",
			"Noto Sans Telugu",
			"Noto Sans Thaana",
			"Noto Sans Thai UI",
			"Noto Sans Thai",
			"Noto Sans Tibetan",
			"Noto Sans Tifinagh",
			"Noto Sans Tirhuta",
			"Noto Sans Ugaritic",
			"Noto Sans Vai",
			"Noto Sans Warang Citi",
			"Noto Sans Yi",
			"Noto Serif Ahom",
			"Noto Serif Armenian",
			"Noto Serif Balinese",
			"Noto Serif Bengali",
			"Noto Serif CJK SC",
			"Noto Serif Devanagari",
			"Noto Serif Display",
			"Noto Serif Ethiopic",
			"Noto Serif Georgian",
			"Noto Serif Gujarati",
			"Noto Serif Gurmukhi",
			"Noto Serif Hebrew",
			"Noto Serif Kannada",
			"Noto Serif Khmer",
			"Noto Serif Lao",
			"Noto Serif Malayalam",
			"Noto Serif Myanmar",
			"Noto Serif Sinhala",
			"Noto Serif Tamil Slanted",
			"Noto Serif Tamil",
			"Noto Serif Telugu",
			"Noto Serif Thai",
			"Noto Serif Tibetan",
			"OpenSymbol",
			"Standard Symbols L",
			"Wingdings",
			/* Some random non-useful stuff from the URW postscript font set
			 * https://github.com/ArtifexSoftware/urw-base35-fonts/tree/master/fonts
			 */
			"C059",
			"D050000L",
			"P052",
			"Z003",
			// FreeBSD specific stuff
			"Cursor",
			"DejaVu Math TeX Gyre",
			"Estrangelo Edessa",
			"Goha-Tibeb Zemen",
			"Bitstream Charter",
			"Noto Kufi Arabic"
		};

		private static readonly List<string> appleBlockedFonts = new List<string>
		{
			".Al Bayan PUA",
			".Al Nile PUA",
			".Al Tarikh PUA",
			".Apple Color Emoji UI",
			".Arabic UI Display",
			".Arabic UI Text",
			".Arial Hebrew Desk Interface",
			".Baghdad PUA",
			".Damascus PUA",
			".DecoType Naskh PUA",
			".Diwan Kufi PUA",
			".Farah PUA",
			".Geeza Pro Interface",
			".Geeza Pro PUA",
			".KufiStandardGK PUA",
			".LastResort",
			".Muna PUA",
			".Nadeem PUA",
			".Noto Nastaliq Urdu UI",
			".Sana PUA",
			"Apple Braille",
			"Apple Color Emoji",
			"Arial Hebrew Scholar",
			"Arial Hebrew",
			"Bodoni Ornaments",
			"Corsiva Hebrew",
			"Devanagari MT",
			"Gujarati MT",
			"Gurmukhi MT",
			"ITF Devanagari Marathi",
			"ITF Devanagari",
			"Kailasa",
			"Kokonor",
			"KufiStandardGK",
			"Mshtakan",
			"New Peninim MT",
			"Noto Nastaliq Urdu",
			"Raanana",
			"STIXIntegralsD",
			"STIXIntegralsSm",
			"STIXIntegralsUp",
			"STIXIntegralsUpD",
			"STIXIntegralsUpSm",
			"STIXNonUnicode",
			"STIXSizeFiveSym",
			"STIXSizeFourSym",
			"STIXSizeOneSym",
			"STIXSizeThreeSym",
			"STIXSizeTwoSym",
			"STIXVariants",
			"Symbol",
			"Webdings",
			"Wingdings 2",
			"Wingdings 3",
			"Wingdings",
			"Zapf Dingbats",
			"البيان",
			"التاريخ",
			"بغداد",
			"بيروت",
			"جيزة",
			"دمشق",
			"ديوان ثلث",
			"ديوان كوفي",
			"صنعاء",
			"فرح",
			"منى",
			"مِصحفي ذهبي",
			"مِصحفي",
			"نسخ",
			"نديم",
		};
	}
}
