#!/usr/bin/env bash
#Set base development branch revision numbers
#This needs to be bumped once we have a stable release branch
MajorVersion=1
MinorVersion=7

# cd to correct directory
cd -P -- "$(dirname -- "$0")"

#If we're a tagged commit
if (git describe --tags --exact-match 2> /dev/null) then
	Version=$(git describe --tags)
	InfoVersion=$(git describe --tags)
else
	# determine revision and build numbers
	if [[ "$OSTYPE" == "darwin"* ]]; then
		#OSX
		Revision=$(((($(date +%s) - $(date -jf "%Y-%m-%d" "2019-10-29" $B +%s))/86400 )+40 ))
		Minutes=$(( $(date "+10#%H * 60 + 10#%M") ))
	else
		#Linux & Cygwin
		Revision=$(( ( ($(date "+%s") - $(date --date="2019-10-29" +%s))/(60*60*24) )+40 ))
		Minutes=$(( ( $(date "+%s") - $(date -d "today 0" +%s))/60 ))
	fi

	Version=$MajorVersion.$MinorVersion.$Revision.$Minutes
	InfoVersion=$MajorVersion.$MinorVersion.$Revision.$Minutes-$USER
fi

cat > AssemblyInfo.cs << EOF

 // This code was generated by a tool. Any changes made manually will be lost
 // the next time this code is regenerated.
 // 
 
using System;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("OpenBVE")]
[assembly: AssemblyProduct("OpenBVE")]
[assembly: AssemblyCopyright("The OpenBVE Project")]
[assembly: ComVisible(false)]
[assembly: AssemblyVersion("$Version")]
[assembly: AssemblyInformationalVersion("$InfoVersion")]
[assembly: AssemblyFileVersion("$Version")]
[assembly: CLSCompliant(false)]
[assembly: InternalsVisibleTo("RouteManager2")]

namespace OpenBve {
	internal static partial class Program {
		internal const string VersionSuffix = "";
	}
}
EOF